#include <math.h>
#include <omp.h>
#include <stdio.h>
#include <stdlib.h>

#include "Ctrl.h"

#define SEED 6834723

#define	DEVICE	0
#define	POLICY	(Ctrl_Policy)1

double main_clock;
double exec_clock;

typedef unsigned char BYTE;

#define N_IMG 3
#define IMG_Y 0
#define IMG_U 1
#define IMG_V 2

Ctrl_NewType(BYTE);

CTRL_KERNEL_CHAR(Sobel_Operation, MANUAL, LOCAL_SIZE_0, LOCAL_SIZE_1);

#define sobel_params	2, OUT, HitTile_BYTE, Output, IN, HitTile_BYTE, Input

#define load_params	4, OUT, HitTile_BYTE, Image_Y, OUT, HitTile_BYTE, Image_U, OUT, HitTile_BYTE, Image_V, INVAL, FILE *, File_reader

#define save_params	4, IN, HitTile_BYTE, Image_Y, IN, HitTile_BYTE, Image_U, IN, HitTile_BYTE, Image_V, INVAL, FILE *, File_writer

#define memset_params	3, OUT, HitTile_BYTE, Image_Y, OUT, HitTile_BYTE, Image_U, OUT, HitTile_BYTE, Image_V

CTRL_KERNEL(Sobel_Operation, GENERIC, DEFAULT, CTRL_KPARAMS( sobel_params ), {
	float Gradient_h;
	float Gradient_v;
	float Gradient_mod;

	unsigned int Col_Index = thread_id_y;
	unsigned int Row_Index = thread_id_x;
	if ((Row_Index != 0) && (Col_Index != 0) && (Row_Index < hit_tileDimCard(Input, 0) - 1) && (Col_Index < hit_tileDimCard(Input, 1) - 1)) {
		Gradient_v = -(-hit(Input, (Row_Index - 1), (Col_Index - 1)) + hit(Input, (Row_Index - 1), (Col_Index + 1)) - 2 * hit(Input, Row_Index, (Col_Index - 1)) + 2 * hit(Input, Row_Index, (Col_Index + 1)) - hit(Input, (Row_Index + 1), (Col_Index - 1)) + hit(Input, (Row_Index + 1), (Col_Index + 1)));

		Gradient_h = -(-hit(Input, (Row_Index - 1), (Col_Index - 1)) - 2 * hit(Input, (Row_Index - 1), Col_Index) - hit(Input, (Row_Index - 1), (Col_Index + 1)) + hit(Input, (Row_Index + 1), (Col_Index - 1)) + 2 * hit(Input, (Row_Index + 1), Col_Index) + hit(Input, (Row_Index + 1), (Col_Index + 1)));

		Gradient_mod = sqrt(Gradient_h * Gradient_h + Gradient_v * Gradient_v);

		hit(Output, Row_Index, Col_Index) = ((int)Gradient_mod < 256) ? (BYTE)Gradient_mod : 255;
	}
});


CTRL_HOST_TASK(Load_Frame, CTRL_HPARAMS( load_params ) ) {
	fread(&(hit(Image_Y, 0)), sizeof(BYTE), hit_tileCard(Image_Y), File_reader);
	fread(&(hit(Image_U, 0)), sizeof(BYTE), hit_tileCard(Image_U), File_reader);
	fread(&(hit(Image_V, 0)), sizeof(BYTE), hit_tileCard(Image_V), File_reader);
}

CTRL_HOST_TASK(Save_Frame, CTRL_HPARAMS( save_params ) ) {
	fwrite(&(hit(Image_Y, 0)), sizeof(BYTE), hit_tileCard(Image_Y), File_writer);
	fwrite(&(hit(Image_U, 0)), sizeof(BYTE), hit_tileCard(Image_U), File_writer);
	fwrite(&(hit(Image_V, 0)), sizeof(BYTE), hit_tileCard(Image_V), File_writer);
}


CTRL_HOST_TASK(Memset, CTRL_HPARAMS( memset_params ) ) {
	memset(&(hit(Image_Y, 0)), 0, hit_tileCard(Image_Y));
	memset(&(hit(Image_U, 0)), 0, hit_tileCard(Image_U));
	memset(&(hit(Image_V, 0)), 0, hit_tileCard(Image_V));
}

CTRL_KERNEL_PROTO(Sobel_Operation, 1, GENERIC, DEFAULT, sobel_params );
CTRL_HOST_TASK_PROTO(Load_Frame, load_params );
CTRL_HOST_TASK_PROTO(Save_Frame, save_params );
CTRL_HOST_TASK_PROTO(Memset, memset_params );

int main(int argc, char *argv[]) {
	main_clock = omp_get_wtime();

	if (argc != 7) {
		printf("Usage: %s <width> <height> <num_frames> <input_yuv_file> <output_yuv_file> <platform>\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	int Width[3];
	Width[0] = atoi(argv[1]);
	Width[1] = Width[2] = Width[0] / 2;

	int Height[3];
	Height[0] = atoi(argv[2]);
	Height[1] = Height[2] = Height[0] / 2;

	int Num_Frames = atoi(argv[3]);

	char       *Input_Filename  = argv[4];
	char       *Output_Filename = argv[5];
	int         PLATFORM        = atoi(argv[6]);

	int Frame_num = 0;

	size_t *sizes = (size_t *)malloc(sizeof(size_t) * N_IMG);
	sizes[IMG_Y]  = (size_t)(Width[IMG_Y] * Height[IMG_Y]);
	sizes[IMG_U]  = (size_t)(Width[IMG_U] * Height[IMG_U]);
	sizes[IMG_V]  = (size_t)(Width[IMG_V] * Height[IMG_V]);

	FILE *File_writer, *File_reader;

	Ctrl_Thread threads[N_IMG];
	for (int i = 0; i < N_IMG; i++) {
		Ctrl_ThreadInit(threads[i], Height[i], Width[i]);
	}

	if (!(File_reader = fopen(Input_Filename, "rb"))) {
		printf("\nError in opening input file: %s\n", Input_Filename);
		exit(EXIT_FAILURE);
	}
	if (!(File_writer = fopen(Output_Filename, "wb+"))) {
		printf("\nError in opening output file: %s\n", Output_Filename);
		exit(EXIT_FAILURE);
	}

	__ctrl_block__(1, 0)
	{
		PCtrl ctrl = Ctrl_Create(CTRL_TYPE_OPENCL_GPU, POLICY, DEVICE, PLATFORM);

		HitTile_BYTE Input_Img[N_IMG];
		HitShape sh1 = hitShapeSize(Height[IMG_Y], Width[IMG_Y]);
		HitShape sh2 = hitShapeSize(Height[IMG_U], Width[IMG_U]);
		HitShape sh3 = hitShapeSize(Height[IMG_V], Width[IMG_V]);

		Input_Img[IMG_Y] = Ctrl_DomainAlloc(ctrl, BYTE, sh1 );
		Input_Img[IMG_U] = Ctrl_DomainAlloc(ctrl, BYTE, sh2 );
		Input_Img[IMG_V] = Ctrl_DomainAlloc(ctrl, BYTE, sh3 );

		HitTile_BYTE Output_Img[N_IMG];
		Output_Img[IMG_Y] = Ctrl_DomainAlloc(ctrl, BYTE, sh1 );
		Output_Img[IMG_U] = Ctrl_DomainAlloc(ctrl, BYTE, sh2 );
		Output_Img[IMG_V] = Ctrl_DomainAlloc(ctrl, BYTE, sh3 );

		Ctrl_HostTask(ctrl, Memset, Output_Img[IMG_Y], Output_Img[IMG_U], Output_Img[IMG_V]);
		Ctrl_MoveTo(ctrl, Output_Img[IMG_Y], Output_Img[IMG_U], Output_Img[IMG_V]);

		Ctrl_GlobalSync(ctrl);
		exec_clock = omp_get_wtime();

		Ctrl_HostTask(ctrl, Load_Frame, Input_Img[IMG_Y], Input_Img[IMG_U], Input_Img[IMG_V], File_reader);

		for (Frame_num = 0; Frame_num < Num_Frames; Frame_num++) {
			for (int i = 0; i < N_IMG; i++) {
				Ctrl_Launch(ctrl, Sobel_Operation, threads[i], CTRL_THREAD_NULL, Output_Img[i], Input_Img[i]);
			}
			if (Frame_num + 1 < Num_Frames) {
				Ctrl_HostTask(ctrl, Load_Frame, Input_Img[IMG_Y], Input_Img[IMG_U], Input_Img[IMG_V], File_reader);
			}
			Ctrl_HostTask(ctrl, Save_Frame, Output_Img[IMG_Y], Output_Img[IMG_U], Output_Img[IMG_V], File_writer);
		}

		Ctrl_GlobalSync(ctrl);
		exec_clock = omp_get_wtime() - exec_clock;

		Ctrl_Free(ctrl, Input_Img[IMG_Y], Input_Img[IMG_U], Input_Img[IMG_V], Output_Img[IMG_Y], Output_Img[IMG_U], Output_Img[IMG_V]);

		Ctrl_Destroy(ctrl);
	}

	free(sizes);
	fclose(File_reader);
	fclose(File_writer);

	main_clock = omp_get_wtime() - main_clock;

	printf("%lf, %lf\n", main_clock, exec_clock);
	return EXIT_SUCCESS;
}
